/*
GNU General Public License

RGBColorizeFilter
Copyright (C) 2000  Howard A Kistler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package com.hexidec.filter;

import java.awt.Color;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageFilter;
import java.awt.image.ImageProducer;
import java.awt.image.RGBImageFilter;

/** RGBColorizeFilter
  * Changes the image by recoloring it with the primary color specified
  *
  * @author Howard Kistler
  * @version 0.1
  *
  * REQUIREMENTS
  * Java 2 (JDK 1.3)
  *
  * VERSION HISTORY
  * 0.1 (10/31/2000) - initial creation (10/16/2000)
  */

public class RGBColorizeFilter extends RGBImageFilter
{
	private float fHSB[] = new float[3];

	float primeHue;        // the Hue of the new primary color
	float primeSaturation; // the Saturation of the new primary color
	float primeBrightness; // the Brightness of the new primary color

//  Constructors ---------------------------------------------------------------------------->

	public RGBColorizeFilter(Color prime)
	{
		canFilterIndexColorModel = true;
		setPrimary(prime);
	}

	public RGBColorizeFilter()
	{
		this(new Color(160, 160, 160));
	}

//  Accessor Method ------------------------------------------------------------------------->

	/** Change the filter's primary color
	  */
	public void setPrimary(Color prime)
	{
		Color.RGBtoHSB(prime.getRed(), prime.getGreen(), prime.getBlue(), fHSB);
		primeHue        = fHSB[0];
		primeSaturation = fHSB[1];
		primeBrightness = fHSB[2];
	}

//  Image Processing Methods ---------------------------------------------------------------->

	/** Modify and return an image using this filter and this color
	  */
	public Image procImage(Image source, Color prime)
	{
		this.setPrimary(prime);
		return procImage(source);
	}

	/** Modify and return an image using this filter and current color
	  */
	public Image procImage(Image source)
	{
		ImageProducer imageProducer = new FilteredImageSource(source.getSource(), this);
		Image recoloredImage = Toolkit.getDefaultToolkit().createImage(imageProducer);
		return recoloredImage;
	}

//  Image Filtering Method ------------------------------------------------------------------>

	/** Called by the filter upon each pixel to determine its modified color.
	  */
	public int filterRGB(int x, int y, int rgb)
	{
		int alpha = (rgb >> 24) & 0xFF;
		int red   = (rgb >> 16) & 0xFF;
		int green = (rgb >>  8) & 0xFF;
		int blue  = (rgb      ) & 0xFF;
		Color.RGBtoHSB(red, green, blue, fHSB);
		float newHue        = primeHue;
		float newSaturation = fHSB[1] * primeSaturation;
		float newBrightness = fHSB[2] * (fHSB[1] * primeBrightness + (1 - fHSB[1]));
		rgb = Color.HSBtoRGB(newHue, newSaturation, newBrightness);
		return (rgb & 0x00FFFFFF) | (alpha << 24);
	}

}